
// -*- mode: c++; c-basic-offset:4 -*-

// This file is part of libdap, A C++ implementation of the OPeNDAP Data
// Access Protocol.

// Copyright (c) 2002,2003 OPeNDAP, Inc.
// Author: James Gallagher <jgallagher@opendap.org>
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// You can contact OPeNDAP, Inc. at PO Box 112, Saunderstown, RI. 02874-0112.
 
// (c) COPYRIGHT URI/MIT 1995-1996,1999
// Please read the full copyright statement in the file COPYRIGHT_URI.
//
// Authors:
//      jhrg,jimg       James Gallagher <jgallagher@gso.uri.edu>

// Implementation for TestList. See TestByte.cc
//
// jhrg 1/12/95

#ifdef __GNUG__
// #pragma implementation
#endif

#include "config_dap.h"

#include <assert.h>

#include "TestList.h"

#ifdef TRACE_NEW
#include "trace_new.h"
#endif

using std::cerr;
using std::endl;

List *
NewList(const string &n, BaseType *v)
{
    return new TestList(n, v);
}

BaseType *
TestList::ptr_duplicate()
{
    return new TestList(*this);
}

TestList::TestList(const string &n, BaseType *v) : List(n, v)
{
}

TestList::~TestList()
{
}

// This `read mfunc' is just like the one for Array, except that it must set
// the List object's length member, since for a list, that value is a
// function of the instance and not of variable. For an array the size of
// thevariable must be declared (in the DDS). However, for a List the size
// will vary depending on the value of the variable (something that is *not*
// declared in the DDS).
//
// NB: we can never have lists of lists ... nor can we have arrays of arrays
// ... In DODS and array is an n-dimensional thing. Lists of lists are to
// amorphous for our translators.
//
// Returns: true.

bool 
TestList::read(const string &dataset)
{
    if (read_p())
	return true;

    // run read() on the contained variable to get, via the read() mfuncs
    // defined in the other Test classes, a value in the *contained* object.
    var()->read(dataset);

    unsigned int elem_wid = var()->width(); // size of an element

    set_length(7);		// this is a poor test; fix it

    unsigned int array_len = length();

    switch (var()->type()) {
      case dods_byte_c:
      case dods_int32_c:
      case dods_uint32_c:
      case dods_float64_c:
      case dods_str_c:
      case dods_url_c: {

	// String and Url are grouped with dods_byte, ... because val2buf works
	// for these types.

	char *tmp = new char[width()];
	// elem_val was a void pointer; delete complained. 6/4/2001 jhrg
	char *elem_val = 0;	// NULL init gets read_val() to alloc space

	// Added cast as temporary fix. 6/4/2001 jhrg
	var()->buf2val((void **)&elem_val); // internal buffer to ELEM_VAL

	for (unsigned i = 0; i < array_len; ++i)
	    memcpy(tmp + i * elem_wid, elem_val, elem_wid);


	val2buf(tmp);

	delete elem_val; elem_val = 0;	// alloced in buf2val()
	delete[] tmp; tmp = 0;	// alloced above

	break;
      }

      case dods_array_c:
      case dods_structure_c:
      case dods_sequence_c:
      case dods_grid_c: {
	
	for (unsigned i = 0; i < array_len; ++i) {
	    BaseType *elem = var()->ptr_duplicate(); 
	    set_vec(i, elem);
	}

	break;
      }

      case dods_list_c:
      case dods_null_c:
      default:
	assert(false);
	cerr << "__FILE__:__LINE__ Bad data type" << endl;
	break;
    }

    set_read_p(true);

    return true;
}

// $Log: TestList.cc,v $
// Revision 1.28.2.1  2004/07/02 20:41:53  jimg
// Removed (commented) the pragma interface/implementation lines. See
// the ChangeLog for more details. This fixes a build problem on HP/UX.
//
// Revision 1.28  2003/04/22 19:40:28  jimg
// Merged with 3.3.1.
//
// Revision 1.27  2003/02/21 00:14:25  jimg
// Repaired copyright.
//
// Revision 1.26.2.1  2003/02/21 00:10:07  jimg
// Repaired copyright.
//
// Revision 1.26  2003/01/23 00:22:24  jimg
// Updated the copyright notice; this implementation of the DAP is
// copyrighted by OPeNDAP, Inc.
//
// Revision 1.25  2001/08/24 17:46:22  jimg
// Resolved conflicts from the merge of release 3.2.6
//
// Revision 1.23.4.3  2001/08/18 00:15:14  jimg
// Removed WIN32 compile guards from using statements.
//
// Revision 1.23.4.2  2001/07/28 01:10:42  jimg
// Some of the numeric type classes did not have copy ctors or operator=.
// I added those where they were needed.
// In every place where delete (or delete []) was called, I set the pointer
// just deleted to zero. Thus if for some reason delete is called again
// before new memory is allocated there won't be a mysterious crash. This is
// just good form when using delete.
// I added calls to www2id and id2www where appropriate. The DAP now handles
// making sure that names are escaped and unescaped as needed. Connect is
// set to handle CEs that contain names as they are in the dataset (see the
// comments/Log there). Servers should not handle escaping or unescaping
// characters on their own.
//
// Revision 1.24  2001/06/15 23:49:02  jimg
// Merged with release-3-2-4.
//
// Revision 1.23.4.1  2001/06/05 06:49:19  jimg
// Added the Constructor class which is to Structures, Sequences and Grids
// what Vector is to Arrays and Lists. This should be used in future
// refactorings (I thought it was going to be used for the back pointers).
// Introduced back pointers so children can refer to their parents in
// hierarchies of variables.
// Added to Sequence methods to tell if a child sequence is done
// deserializing its data.
// Fixed the operator=() and copy ctors; removed redundency from
// _duplicate().
// Changed the way serialize and deserialize work for sequences. Now SOI and
// EOS markers are written for every `level' of a nested Sequence. This
// should fixed nested Sequences. There is still considerable work to do
// for these to work in all cases.
//
// Revision 1.23  2000/09/21 16:22:09  jimg
// Merged changes from Jose Garcia that add exceptions to the software.
// Many methods that returned error codes now throw exectptions. There are
// two classes which are thrown by the software, Error and InternalErr.
// InternalErr is used to report errors within the library or errors using
// the library. Error is used to reprot all other errors. Since InternalErr
// is a subclass of Error, programs need only to catch Error.
//
// Revision 1.22  2000/07/09 22:05:36  rmorris
// Changes to increase portability, minimize ifdef's for win32 and account
// for differences in the iostreams implementations.
//
// Revision 1.21.14.1  2000/02/17 05:03:15  jimg
// Added file and line number information to calls to InternalErr.
// Resolved compile-time problems with read due to a change in its
// parameter list given that errors are now reported using exceptions.
//
// Revision 1.21  1999/05/04 19:47:22  jimg
// Fixed copyright statements. Removed more of the GNU classes.
//
// Revision 1.20  1999/04/29 02:29:32  jimg
// Merge of no-gnu branch
//
// Revision 1.19  1998/11/10 00:57:14  jimg
// Fixed errant delete [] (should have been plain delete).
//
// Revision 1.18.14.1  1999/02/02 21:57:03  jimg
// String to string version
//
// Revision 1.18  1996/12/02 18:21:19  jimg
// Added case for unit32 to ops() member functon.
//
// Revision 1.17  1996/08/13 20:50:45  jimg
// Changed definition of the read member function.
//
// Revision 1.16  1996/05/31 23:30:26  jimg
// Updated copyright notice.
//
// Revision 1.15  1996/05/22 18:05:26  jimg
// Merged files from the old netio directory into the dap directory.
// Removed the errmsg library from the software.
//
// Revision 1.14  1996/05/14 15:38:42  jimg
// These changes have already been checked in once before. However, I
// corrupted the source repository and restored it from a 5/9/96 backup
// tape. The previous version's log entry should cover the changes.
//
// Revision 1.13  1996/04/05 21:59:30  jimg
// Misc Changes for release 2.0.1 of the core software - for developers.
//
// Revision 1.12  1996/04/05 00:21:56  jimg
// Compiled with g++ -Wall and fixed various warnings.
//
// Revision 1.11  1996/03/05 18:51:30  jimg
// Fixed variable scoping problems in switch and for statements.
//
// Revision 1.10  1995/12/09  01:07:20  jimg
// Added changes so that relational operators will work properly for all the
// datatypes (including Sequences). The relational ops are evaluated in
// DDS::eval_constraint() after being parsed by DDS::parse_constraint().
//
// Revision 1.9  1995/12/06  19:55:25  jimg
// Changes read() member function from three arguments to two.
//
// Revision 1.8  1995/08/23  00:49:59  jimg
// Fixed the read() member function so that it works correctly for arrays/lists
// of Structure, ... types.
//
// Revision 1.7  1995/07/09  21:29:17  jimg
// Added copyright notice.
//
// Revision 1.6  1995/05/10  13:45:35  jimg
// Changed the name of the configuration header file from `config.h' to
// `config_dap.h' so that other libraries could have header files which were
// installed in the DODS include directory without overwriting this one. Each
// config header should follow the convention config_<name>.h.
//
// Revision 1.5  1995/03/16  17:33:52  jimg
// The class List is still hopelessly broken... This is checked in only so
// that the rest of the library will compile.
//
// Revision 1.4  1995/03/04  14:38:07  jimg
// Modified these so that they fit with the changes in the DAP classes.
//
// Revision 1.3  1995/02/10  02:33:45  jimg
// Modified Test<class>.h and .cc so that they used to new definitions of
// read_val().
// Modified the classes read() so that they are more in line with the
// class library's intended use in a real subclass set.
//
// Revision 1.2  1995/01/19  21:58:58  jimg
// Added read_val from dummy_read.cc to the sample set of sub-class
// implementations.
// Changed the declaration of readVal in BaseType so that it names the
// mfunc read_val (to be consistant with the other mfunc names).
// Removed the unnecessary duplicate declaration of the abstract virtual
// mfuncs read and (now) read_val from the classes Byte, ... Grid. The
// declaration in BaseType is sufficient along with the decl and definition
// in the *.cc,h files which contain the subclasses for Byte, ..., Grid.
//
// Revision 1.1  1995/01/19  20:20:49  jimg
// Created as an example of subclassing the class hierarchy rooted at
// BaseType.
//
